@@ -1,10 +1,10 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1997-2003
+ * Copyright (c) 1997-2001
  *	Sleepycat Software.  All rights reserved.
  *
- * $Id: db_cxx.in,v 11.134 2003/09/04 19:02:27 bostic Exp $
+ * $Id: db_cxx.in,v 11.87 2001/11/09 21:31:35 bostic Exp $
  */
 
 #ifndef _DB_CXX_H_
@@ -54,13 +54,15 @@
 #define	HAVE_CXX_STDHEADERS 1
 #ifdef HAVE_CXX_STDHEADERS
 #include <iostream>
-#define	__DB_OSTREAMCLASS	std::ostream
+#define	OSTREAMCLASS	std::ostream
 #else
 #include <iostream.h>
-#define	__DB_OSTREAMCLASS	ostream
+#define	OSTREAMCLASS	ostream
 #endif
 
 #include "db.h"
+#include "cxx_common.h"
+#include "cxx_except.h"
 
 class Db;                                        // forward
 class Dbc;                                       // forward
@@ -70,20 +72,8 @@ class DbLock;                                    // forward
 class DbLogc;                                    // forward
 class DbLsn;                                     // forward
 class DbMpoolFile;                               // forward
-class DbPreplist;                                // forward
 class Dbt;                                       // forward
 class DbTxn;                                     // forward
-class DbDeadlockException;                       // forward
-class DbException;                               // forward
-class DbLockNotGrantedException;                 // forward
-class DbLock;                                    // forward
-class DbMemoryException;                         // forward
-class DbRunRecoveryException;                    // forward
-class Dbt;                                       // forward
-class DbMultipleIterator;                        // forward
-class DbMultipleKeyDataIterator;                 // forward
-class DbMultipleRecnoDataIterator;               // forward
-class DbMultipleDataIterator;                    // forward
 
 // These classes are not defined here and should be invisible
 // to the user, but some compilers require forward references.
@@ -131,35 +121,6 @@ class DbTxnImp;
 
 #endif
 
-////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////
-//
-// Mechanisms for declaring classes
-//
-
-//
-// Every class defined in this file has an _exported next to the class name.
-// This is needed for WinTel machines so that the class methods can
-// be exported or imported in a DLL as appropriate.  Users of the DLL
-// use the define DB_USE_DLL.  When the DLL is built, DB_CREATE_DLL
-// must be defined.
-//
-#if defined(_MSC_VER)
-
-#  if defined(DB_CREATE_DLL)
-#    define _exported __declspec(dllexport)      // creator of dll
-#  elif defined(DB_USE_DLL)
-#    define _exported __declspec(dllimport)      // user of dll
-#  else
-#    define _exported                            // static lib creator or user
-#  endif
-
-#else /* _MSC_VER */
-
-#  define _exported
-
-#endif /* _MSC_VER */
-
 // Some interfaces can be customized by allowing users to define
 // callback functions.  For performance and logistical reasons, some
 // callback functions must be declared in extern "C" blocks.  For others,
@@ -188,122 +149,6 @@ extern "C" {
 		(DB_ENV *dbenv, db_pgno_t pgno, void *pgaddr, DBT *pgcookie);
 };
 
-////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////
-//
-// Exception classes
-//
-
-// Almost any error in the DB library throws a DbException.
-// Every exception should be considered an abnormality
-// (e.g. bug, misuse of DB, file system error).
-//
-// NOTE: We would like to inherit from class exception and
-//       let it handle what(), but there are
-//       MSVC++ problems when <exception> is included.
-//
-class _exported DbException
-{
-public:
-	virtual ~DbException();
-	DbException(int err);
-	DbException(const char *description);
-	DbException(const char *prefix, int err);
-	DbException(const char *prefix1, const char *prefix2, int err);
-	int get_errno() const;
-	virtual const char *what() const;
-	DbEnv *get_env() const;
-	void set_env(DbEnv *env);
-
-	DbException(const DbException &);
-	DbException &operator = (const DbException &);
-
-private:
-	char *what_;
-	int err_;                   // errno
-	DbEnv *env_;
-};
-
-//
-// A specific sort of exception that occurs when
-// an operation is aborted to resolve a deadlock.
-//
-class _exported DbDeadlockException : public DbException
-{
-public:
-	virtual ~DbDeadlockException();
-	DbDeadlockException(const char *description);
-
-	DbDeadlockException(const DbDeadlockException &);
-	DbDeadlockException &operator = (const DbDeadlockException &);
-};
-
-//
-// A specific sort of exception that occurs when
-// a lock is not granted, e.g. by lock_get or lock_vec.
-// Note that the Dbt is only live as long as the Dbt used
-// in the offending call.
-//
-class _exported DbLockNotGrantedException : public DbException
-{
-public:
-	virtual ~DbLockNotGrantedException();
-	DbLockNotGrantedException(const char *prefix, db_lockop_t op,
-	    db_lockmode_t mode, const Dbt *obj, const DbLock lock, int index);
-	DbLockNotGrantedException(const char *description);
-	DbLockNotGrantedException(const DbLockNotGrantedException &);
-	DbLockNotGrantedException &operator =
-	    (const DbLockNotGrantedException &);
-
-	db_lockop_t get_op() const;
-	db_lockmode_t get_mode() const;
-	const Dbt* get_obj() const;
-	DbLock *get_lock() const;
-	int get_index() const;
-
-private:
-	db_lockop_t op_;
-	db_lockmode_t mode_;
-	const Dbt *obj_;
-	DbLock *lock_;
-	int index_;
-};
-
-//
-// A specific sort of exception that occurs when
-// user declared memory is insufficient in a Dbt.
-//
-class _exported DbMemoryException : public DbException
-{
-public:
-	virtual ~DbMemoryException();
-	DbMemoryException(Dbt *dbt);
-	DbMemoryException(const char *description);
-	DbMemoryException(const char *prefix, Dbt *dbt);
-	DbMemoryException(const char *prefix1, const char *prefix2, Dbt *dbt);
-	Dbt *get_dbt() const;
-
-	DbMemoryException(const DbMemoryException &);
-	DbMemoryException &operator = (const DbMemoryException &);
-
-private:
-	Dbt *dbt_;
-};
-
-//
-// A specific sort of exception that occurs when
-// recovery is required before continuing DB activity.
-//
-class _exported DbRunRecoveryException : public DbException
-{
-public:
-	virtual ~DbRunRecoveryException();
-	DbRunRecoveryException(const char *description);
-
-	DbRunRecoveryException(const DbRunRecoveryException &);
-	DbRunRecoveryException &operator = (const DbRunRecoveryException &);
-};
-
 ////////////////////////////////////////////////////////////////
 ////////////////////////////////////////////////////////////////
 //
@@ -316,6 +161,9 @@ class _exported DbLock
 
 public:
 	DbLock();
+
+	int put(DbEnv *env);
+
 	DbLock(const DbLock &);
 	DbLock &operator = (const DbLock &);
 
@@ -334,7 +182,7 @@ protected:
 // Log classes
 //
 
-class _exported DbLsn : public DB_LSN
+class _exported DbLsn : protected DB_LSN
 {
 	friend class DbEnv;          // friendship needed to cast to base class
 	friend class DbLogc;         // friendship needed to cast to base class
@@ -349,46 +197,23 @@ class _exported DbLsn : public DB_LSN
 class _exported DbMpoolFile
 {
 	friend class DbEnv;
-	friend class Db;
-
-private:
-	// Put this first to allow inlining with some C++ compilers (g++-2.95)
-	DEFINE_DB_CLASS(DbMpoolFile);
 
 public:
 	int close(u_int32_t flags);
 	int get(db_pgno_t *pgnoaddr, u_int32_t flags, void *pagep);
+	void last_pgno(db_pgno_t *pgnoaddr);
 	int open(const char *file, u_int32_t flags, int mode, size_t pagesize);
 	int put(void *pgaddr, u_int32_t flags);
+	void refcnt(db_pgno_t *pgnoaddr);
 	int set(void *pgaddr, u_int32_t flags);
-	int get_clear_len(u_int32_t *len);
 	int set_clear_len(u_int32_t len);
-	int get_fileid(u_int8_t *fileid);
 	int set_fileid(u_int8_t *fileid);
-	int get_flags(u_int32_t *flagsp);
-	int set_flags(u_int32_t flags, int onoff);
-	int get_ftype(int *ftype);
 	int set_ftype(int ftype);
-	int get_lsn_offset(int32_t *offsetp);
 	int set_lsn_offset(int32_t offset);
-	int get_maxsize(u_int32_t *gbytes, u_int32_t *bytes);
-	int set_maxsize(u_int32_t gbytes, u_int32_t bytes);
-	int get_pgcookie(DBT *dbt);
 	int set_pgcookie(DBT *dbt);
-	int get_priority(DB_CACHE_PRIORITY *priorityp);
-	int set_priority(DB_CACHE_PRIORITY priority);
+	void set_unlink(int);
 	int sync();
 
-	virtual DB_MPOOLFILE *get_DB_MPOOLFILE()
-	{
-		return (DB_MPOOLFILE *)imp();
-	}
-
-	virtual const DB_MPOOLFILE *get_const_DB_MPOOLFILE() const
-	{
-		return (const DB_MPOOLFILE *)constimp();
-	}
-
 private:
 	// We can add data to this class if needed
 	// since it is implemented via a pointer.
@@ -401,25 +226,14 @@ private:
 
 	// Shut g++ up.
 protected:
-	virtual ~DbMpoolFile();
+	~DbMpoolFile();
 
 private:
 	// no copying
 	DbMpoolFile(const DbMpoolFile &);
 	void operator = (const DbMpoolFile &);
-};
 
-////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////
-//
-// This is filled in and returned by the DbEnv::txn_recover() method.
-//
-
-class _exported DbPreplist
-{
-public:
-	DbTxn *txn;
-	u_int8_t gid[DB_XIDDATASIZE];
+	DEFINE_DB_CLASS(DbMpoolFile);
 };
 
 ////////////////////////////////////////////////////////////////
@@ -432,41 +246,13 @@ class _exported DbTxn
 {
 	friend class DbEnv;
 
-private:
-	// Put this first to allow inlining with some C++ compilers (g++-2.95)
-	DEFINE_DB_CLASS(DbTxn);
-
 public:
 	int abort();
 	int commit(u_int32_t flags);
-	int discard(u_int32_t flags);
 	u_int32_t id();
 	int prepare(u_int8_t *gid);
 	int set_timeout(db_timeout_t timeout, u_int32_t flags);
 
-	virtual DB_TXN *get_DB_TXN()
-	{
-		return (DB_TXN *)imp();
-	}
-
-	virtual const DB_TXN *get_const_DB_TXN() const
-	{
-		return (const DB_TXN *)constimp();
-	}
-
-	static DbTxn* get_DbTxn(DB_TXN *txn)
-	{
-		return (DbTxn *)txn->api_internal;
-	}
-
-	static const DbTxn* get_const_DbTxn(const DB_TXN *txn)
-	{
-		return (const DbTxn *)txn->api_internal;
-	}
-
-	// For internal use only.
-	static DbTxn* wrap_DB_TXN(DB_TXN *txn);
-
 private:
 	// We can add data to this class if needed
 	// since it is implemented via a pointer.
@@ -477,13 +263,13 @@ private:
 	// delete to release them.
 	//
 	DbTxn();
-	// For internal use only.
-	DbTxn(DB_TXN *txn);
-	virtual ~DbTxn();
+	~DbTxn();
 
 	// no copying
 	DbTxn(const DbTxn &);
 	void operator = (const DbTxn &);
+
+	DEFINE_DB_CLASS(DbTxn);
 };
 
 //
@@ -501,11 +287,10 @@ class _exported DbEnv
 	friend class DbLock;
 	friend class DbMpoolFile;
 
-private:
-	// Put this first to allow inlining with some C++ compilers (g++-2.95)
-	DEFINE_DB_CLASS(DbEnv);
-
 public:
+
+	~DbEnv();
+
 	// After using this constructor, you can set any needed
 	// parameters for the environment using the set_* methods.
 	// Then call open() to finish initializing the environment
@@ -513,78 +298,46 @@ public:
 	//
 	DbEnv(u_int32_t flags);
 
-	virtual ~DbEnv();
-
 	// These methods match those in the C interface.
 	//
-	virtual int close(u_int32_t);
-	virtual int dbremove(DbTxn *txn, const char *name, const char *subdb,
-	    u_int32_t flags);
-	virtual int dbrename(DbTxn *txn, const char *name, const char *subdb,
-	    const char *newname, u_int32_t flags);
-	virtual void err(int, const char *, ...);
-	virtual void errx(const char *, ...);
-	virtual void *get_app_private() const;
-	virtual int get_home(const char **);
-	virtual int get_open_flags(u_int32_t *);
-	virtual int open(const char *, u_int32_t, int);
-	virtual int remove(const char *, u_int32_t);
-	virtual int set_alloc(db_malloc_fcn_type, db_realloc_fcn_type,
+	int close(u_int32_t);
+	void err(int, const char *, ...);
+	void errx(const char *, ...);
+	void *get_app_private() const;
+	int open(const char *, u_int32_t, int);
+	int remove(const char *, u_int32_t);
+	int set_alloc(db_malloc_fcn_type, db_realloc_fcn_type,
 		      db_free_fcn_type);
-	virtual void set_app_private(void *);
-	virtual int get_cachesize(u_int32_t *, u_int32_t *, int *);
-	virtual int set_cachesize(u_int32_t, u_int32_t, int);
-	virtual int get_data_dirs(const char ***);
-	virtual int set_data_dir(const char *);
-	virtual int get_encrypt_flags(u_int32_t *);
-	virtual int set_encrypt(const char *, u_int32_t);
-	virtual void set_errcall(void (*)(const char *, char *));
-	virtual void get_errfile(FILE **);
-	virtual void set_errfile(FILE *);
-	virtual void get_errpfx(const char **);
-	virtual void set_errpfx(const char *);
-	virtual int get_flags(u_int32_t *);
-	virtual int set_flags(u_int32_t, int);
-	virtual int set_feedback(void (*)(DbEnv *, int, int));
-	virtual int get_lg_bsize(u_int32_t *);
-	virtual int set_lg_bsize(u_int32_t);
-	virtual int get_lg_dir(const char **);
-	virtual int set_lg_dir(const char *);
-	virtual int get_lg_max(u_int32_t *);
-	virtual int set_lg_max(u_int32_t);
-	virtual int get_lg_regionmax(u_int32_t *);
-	virtual int set_lg_regionmax(u_int32_t);
-	virtual int get_lk_conflicts(const u_int8_t **, int *);
-	virtual int set_lk_conflicts(u_int8_t *, int);
-	virtual int get_lk_detect(u_int32_t *);
-	virtual int set_lk_detect(u_int32_t);
-	virtual int set_lk_max(u_int32_t);
-	virtual int get_lk_max_lockers(u_int32_t *);
-	virtual int set_lk_max_lockers(u_int32_t);
-	virtual int get_lk_max_locks(u_int32_t *);
-	virtual int set_lk_max_locks(u_int32_t);
-	virtual int get_lk_max_objects(u_int32_t *);
-	virtual int set_lk_max_objects(u_int32_t);
-	virtual int get_mp_mmapsize(size_t *);
-	virtual int set_mp_mmapsize(size_t);
-	virtual int set_paniccall(void (*)(DbEnv *, int));
-	virtual int set_rpc_server(void *, char *, long, long, u_int32_t);
-	virtual int get_shm_key(long *);
-	virtual int set_shm_key(long);
-	virtual int get_timeout(db_timeout_t *, u_int32_t);
-	virtual int set_timeout(db_timeout_t, u_int32_t);
-	virtual int get_tmp_dir(const char **);
-	virtual int set_tmp_dir(const char *);
-	virtual int get_tas_spins(u_int32_t *);
-	virtual int set_tas_spins(u_int32_t);
-	virtual int get_tx_max(u_int32_t *);
-	virtual int set_tx_max(u_int32_t);
-	virtual int set_app_dispatch(int (*)(DbEnv *,
-	    Dbt *, DbLsn *, db_recops));
-	virtual int get_tx_timestamp(time_t *);
-	virtual int set_tx_timestamp(time_t *);
-	virtual int get_verbose(u_int32_t which, int *);
-	virtual int set_verbose(u_int32_t which, int);
+	void set_app_private(void *);
+	int set_cachesize(u_int32_t, u_int32_t, int);
+	int set_data_dir(const char *);
+	void set_errcall(void (*)(const char *, char *));
+	void set_errfile(FILE *);
+	void set_errpfx(const char *);
+	int set_flags(u_int32_t, int);
+	int set_feedback(void (*)(DbEnv *, int, int));
+	int set_recovery_init(int (*)(DbEnv *));
+	int set_lg_bsize(u_int32_t);
+	int set_lg_dir(const char *);
+	int set_lg_max(u_int32_t);
+	int set_lg_regionmax(u_int32_t);
+	int set_lk_conflicts(u_int8_t *, int);
+	int set_lk_detect(u_int32_t);
+	int set_lk_max(u_int32_t);
+	int set_lk_max_lockers(u_int32_t);
+	int set_lk_max_locks(u_int32_t);
+	int set_lk_max_objects(u_int32_t);
+	int set_mp_mmapsize(size_t);
+	int set_paniccall(void (*)(DbEnv *, int));
+	int set_rpc_server(void *, char *, long, long, u_int32_t);
+	int set_shm_key(long);
+	int set_timeout(db_timeout_t timeout, u_int32_t flags);
+	int set_tmp_dir(const char *);
+	int set_tas_spins(u_int32_t);
+	int set_tx_max(u_int32_t);
+	int set_tx_recover(int (*)(DbEnv *, Dbt *, DbLsn *, db_recops));
+	int set_tx_timestamp(time_t *);
+	int set_verbose(u_int32_t which, int onoff);
 
 	// Version information.  A static method so it can be obtained anytime.
 	//
@@ -602,138 +355,132 @@ public:
 	// set_error_stream() to force all errors to a C++ stream.
 	// It is unwise to mix these approaches.
 	//
-	virtual void set_error_stream(__DB_OSTREAMCLASS *);
+	void set_error_stream(OSTREAMCLASS *);
 
 	// used internally
-	static void runtime_error(DbEnv *env, const char *caller, int err,
+	static void runtime_error(const char *caller, int err,
 				  int error_policy);
-	static void runtime_error_dbt(DbEnv *env, const char *caller, Dbt *dbt,
-				  int error_policy);
-	static void runtime_error_lock_get(DbEnv *env, const char *caller,
-				  int err, db_lockop_t op, db_lockmode_t mode,
-				  const Dbt *obj, DbLock lock, int index,
+	static void runtime_error_dbt(const char *caller, Dbt *dbt,
 				  int error_policy);
 
 	// Lock functions
 	//
-	virtual int lock_detect(u_int32_t flags, u_int32_t atype, int *aborted);
-	virtual int lock_get(u_int32_t locker, u_int32_t flags, const Dbt *obj,
+	int lock_detect(u_int32_t flags, u_int32_t atype, int *aborted);
+	int lock_get(u_int32_t locker, u_int32_t flags, const Dbt *obj,
 		     db_lockmode_t lock_mode, DbLock *lock);
-	virtual int lock_id(u_int32_t *idp);
-	virtual int lock_id_free(u_int32_t id);
-	virtual int lock_put(DbLock *lock);
-	virtual int lock_stat(DB_LOCK_STAT **statp, u_int32_t flags);
-	virtual int lock_vec(u_int32_t locker, u_int32_t flags,
-		     DB_LOCKREQ list[], int nlist, DB_LOCKREQ **elistp);
+	int lock_id(u_int32_t *idp);
+	int lock_id_free(u_int32_t id);
+	int lock_stat(DB_LOCK_STAT **statp, u_int32_t flags);
+	int lock_vec(u_int32_t locker, u_int32_t flags, DB_LOCKREQ list[],
+		     int nlist, DB_LOCKREQ **elistp);
 
 	// Log functions
 	//
-	virtual int log_archive(char **list[], u_int32_t flags);
+	int log_archive(char **list[], u_int32_t flags);
 	static int log_compare(const DbLsn *lsn0, const DbLsn *lsn1);
-	virtual int log_cursor(DbLogc **cursorp, u_int32_t flags);
-	virtual int log_file(DbLsn *lsn, char *namep, size_t len);
-	virtual int log_flush(const DbLsn *lsn);
-	virtual int log_put(DbLsn *lsn, const Dbt *data, u_int32_t flags);
+	int log_cursor(DbLogc **cursorp, u_int32_t flags);
+	int log_file(DbLsn *lsn, char *namep, size_t len);
+	int log_flush(const DbLsn *lsn);
+	int log_put(DbLsn *lsn, const Dbt *data, u_int32_t flags);
 
-	virtual int log_stat(DB_LOG_STAT **spp, u_int32_t flags);
+	int log_register(Db *dbp, const char *name);
+	int log_stat(DB_LOG_STAT **spp, u_int32_t flags);
+	int log_unregister(Db *dbp);
 
 	// Mpool functions
 	//
-	virtual int memp_fcreate(DbMpoolFile **dbmfp, u_int32_t flags);
-	virtual int memp_register(int ftype,
+	int memp_fcreate(DbMpoolFile **dbmfp, u_int32_t flags);
+	int memp_register(int ftype,
 			  pgin_fcn_type pgin_fcn,
 			  pgout_fcn_type pgout_fcn);
-	virtual int memp_stat(DB_MPOOL_STAT
+	int memp_stat(DB_MPOOL_STAT
 		      **gsp, DB_MPOOL_FSTAT ***fsp, u_int32_t flags);
-	virtual int memp_sync(DbLsn *lsn);
-	virtual int memp_trickle(int pct, int *nwrotep);
+	int memp_sync(DbLsn *lsn);
+	int memp_trickle(int pct, int *nwrotep);
 
 	// Transaction functions
 	//
-	virtual int txn_begin(DbTxn *pid, DbTxn **tid, u_int32_t flags);
-	virtual int txn_checkpoint(u_int32_t kbyte, u_int32_t min,
-			u_int32_t flags);
-	virtual int txn_recover(DbPreplist *preplist, long count,
+	int txn_begin(DbTxn *pid, DbTxn **tid, u_int32_t flags);
+	int txn_checkpoint(u_int32_t kbyte, u_int32_t min, u_int32_t flags);
+	int txn_recover(DB_PREPLIST *preplist, long count,
 			long *retp, u_int32_t flags);
-	virtual int txn_stat(DB_TXN_STAT **statp, u_int32_t flags);
+	int txn_stat(DB_TXN_STAT **statp, u_int32_t flags);
 
 	// Replication functions
 	//
-	virtual int rep_elect(int, int, u_int32_t, int *);
-	virtual int rep_process_message(Dbt *, Dbt *, int *, DbLsn *);
-	virtual int rep_start(Dbt *, u_int32_t);
-	virtual int rep_stat(DB_REP_STAT **statp, u_int32_t flags);
-	virtual int get_rep_limit(u_int32_t *, u_int32_t *);
-	virtual int set_rep_limit(u_int32_t, u_int32_t);
-	virtual int set_rep_transport(int, int (*)(DbEnv *,
-	    const Dbt *, const Dbt *, const DbLsn *, int, u_int32_t));
+	int rep_elect(int, int, u_int32_t, int *);
+	int rep_process_message(Dbt *, Dbt *, int *);
+	int rep_start(Dbt *, u_int32_t);
+	int set_rep_transport(u_int32_t,
+	    int (*)(DbEnv *, const Dbt *, const Dbt *, int, u_int32_t));
 
 	// Conversion functions
 	//
-	virtual DB_ENV *get_DB_ENV()
+	DB_ENV *get_DB_ENV()
 	{
 		return (DB_ENV *)imp();
 	}
 
-	virtual const DB_ENV *get_const_DB_ENV() const
+	const DB_ENV *get_const_DB_ENV() const
 	{
 		return (const DB_ENV *)constimp();
 	}
 
 	static DbEnv* get_DbEnv(DB_ENV *dbenv)
 	{
-		return (DbEnv *)dbenv->api1_internal;
+		return (DbEnv *)dbenv->cj_internal;
 	}
 
 	static const DbEnv* get_const_DbEnv(const DB_ENV *dbenv)
 	{
-		return (const DbEnv *)dbenv->api1_internal;
+		return (const DbEnv *)dbenv->cj_internal;
 	}
 
-	// For internal use only.
-	static DbEnv* wrap_DB_ENV(DB_ENV *dbenv);
-
 	// These are public only because they need to be called
 	// via C functions.  They should never be called by users
 	// of this class.
 	//
 	static void _stream_error_function(const char *, char *);
-	static int _app_dispatch_intercept(DB_ENV *env, DBT *dbt, DB_LSN *lsn,
+	static int _tx_recover_intercept(DB_ENV *env, DBT *dbt, DB_LSN *lsn,
 					db_recops op);
 	static void _paniccall_intercept(DB_ENV *env, int errval);
+	static int _recovery_init_intercept(DB_ENV *env);
 	static void _feedback_intercept(DB_ENV *env, int opcode, int pct);
 	static int _rep_send_intercept(DB_ENV *env,
 				       const DBT *cntrl, const DBT *data,
-				       const DB_LSN *lsn, int id,
-				       u_int32_t flags);
+				       int id, u_int32_t flags);
+	static void _destroy_check(const char *str, int isDbEnv);
 
 private:
 	void cleanup();
 	int initialize(DB_ENV *env);
 	int error_policy();
 
-	// For internal use only.
+	// Used internally
 	DbEnv(DB_ENV *, u_int32_t flags);
 
 	// no copying
 	DbEnv(const DbEnv &);
 	void operator = (const DbEnv &);
 
+	DEFINE_DB_CLASS(DbEnv);
+
 	// instance data
 	int construct_error_;
 	u_int32_t construct_flags_;
-	int (*app_dispatch_callback_)(DbEnv *, Dbt *, DbLsn *, db_recops);
-	void (*feedback_callback_)(DbEnv *, int, int);
+	int (*tx_recover_callback_)(DbEnv *, Dbt *, DbLsn *, db_recops);
+	int (*recovery_init_callback_)(DbEnv *);
 	void (*paniccall_callback_)(DbEnv *, int);
+	void (*feedback_callback_)(DbEnv *, int, int);
 	int (*pgin_callback_)(DbEnv *dbenv, db_pgno_t pgno,
 			      void *pgaddr, Dbt *pgcookie);
 	int (*pgout_callback_)(DbEnv *dbenv, db_pgno_t pgno,
 			       void *pgaddr, Dbt *pgcookie);
 	int (*rep_send_callback_)(DbEnv *,
-	    const Dbt *, const Dbt *, const DbLsn *, int, u_int32_t);
+	    const Dbt *, const Dbt *, int, u_int32_t);
 
 	// class data
-	static __DB_OSTREAMCLASS *error_stream_;
+	static OSTREAMCLASS *error_stream_;
 };
 
 ////////////////////////////////////////////////////////////////
@@ -749,149 +496,121 @@ class _exported Db
 {
 	friend class DbEnv;
 
-private:
-	// Put this first to allow inlining with some C++ compilers (g++-2.95)
-	DEFINE_DB_CLASS(Db);
-
 public:
 	Db(DbEnv*, u_int32_t);      // create a Db object, then call open()
-	virtual ~Db();              // does *not* call close.
+	~Db();                      // does *not* call close.
 
 	// These methods exactly match those in the C interface.
 	//
-	virtual int associate(DbTxn *txn, Db *secondary,
-	    int (*callback)(Db *, const Dbt *, const Dbt *, Dbt *),
-	    u_int32_t flags);
-	virtual int close(u_int32_t flags);
-	virtual int cursor(DbTxn *txnid, Dbc **cursorp, u_int32_t flags);
-	virtual int del(DbTxn *txnid, Dbt *key, u_int32_t flags);
-	virtual void err(int, const char *, ...);
-	virtual void errx(const char *, ...);
-	virtual int fd(int *fdp);
-	virtual int get(DbTxn *txnid, Dbt *key, Dbt *data, u_int32_t flags);
-	virtual void *get_app_private() const;
-	virtual int get_byteswapped(int *);
-	virtual int get_dbname(const char **, const char **);
-	virtual int get_open_flags(u_int32_t *);
-	virtual int get_type(DBTYPE *);
-	virtual int join(Dbc **curslist, Dbc **dbcp, u_int32_t flags);
-	virtual int key_range(DbTxn *, Dbt *, DB_KEY_RANGE *, u_int32_t);
-	virtual int open(DbTxn *txnid,
-	    const char *, const char *subname, DBTYPE, u_int32_t, int);
-	virtual int pget(DbTxn *txnid, Dbt *key, Dbt *pkey, Dbt *data,
+	int associate(Db *secondary, int (*callback)(Db *, const Dbt *,
+		const Dbt *, Dbt *), u_int32_t flags);
+	int close(u_int32_t flags);
+	int cursor(DbTxn *txnid, Dbc **cursorp, u_int32_t flags);
+	int del(DbTxn *txnid, Dbt *key, u_int32_t flags);
+	void err(int, const char *, ...);
+	void errx(const char *, ...);
+	int fd(int *fdp);
+	int get(DbTxn *txnid, Dbt *key, Dbt *data, u_int32_t flags);
+	void *get_app_private() const;
+	int get_byteswapped(int *);
+	int get_type(DBTYPE *);
+	int join(Dbc **curslist, Dbc **dbcp, u_int32_t flags);
+	int key_range(DbTxn *, Dbt *, DB_KEY_RANGE *, u_int32_t);
+	int open(const char *, const char *subname, DBTYPE, u_int32_t, int);
+	int pget(DbTxn *txnid, Dbt *key, Dbt *pkey, Dbt *data,
 		 u_int32_t flags);
-	virtual int put(DbTxn *, Dbt *, Dbt *, u_int32_t);
-	virtual int remove(const char *, const char *, u_int32_t);
-	virtual int rename(const char *, const char *, const char *, u_int32_t);
-	virtual int set_alloc(db_malloc_fcn_type, db_realloc_fcn_type,
+	int put(DbTxn *, Dbt *, Dbt *, u_int32_t);
+	int remove(const char *, const char *, u_int32_t);
+	int rename(const char *, const char *, const char *, u_int32_t);
+	int set_alloc(db_malloc_fcn_type, db_realloc_fcn_type,
 		      db_free_fcn_type);
-	virtual void set_app_private(void *);
-	virtual int set_append_recno(int (*)(Db *, Dbt *, db_recno_t));
-	virtual int set_bt_compare(bt_compare_fcn_type); /*deprecated*/
-	virtual int set_bt_compare(int (*)(Db *, const Dbt *, const Dbt *));
-	virtual int set_bt_maxkey(u_int32_t);
-	virtual int get_bt_minkey(u_int32_t *);
-	virtual int set_bt_minkey(u_int32_t);
-	virtual int set_bt_prefix(bt_prefix_fcn_type); /*deprecated*/
-	virtual int set_bt_prefix(size_t (*)(Db *, const Dbt *, const Dbt *));
-	virtual int get_cachesize(u_int32_t *, u_int32_t *, int *);
-	virtual int set_cachesize(u_int32_t, u_int32_t, int);
-	virtual int set_dup_compare(dup_compare_fcn_type); /*deprecated*/
-	virtual int set_dup_compare(int (*)(Db *, const Dbt *, const Dbt *));
-	virtual int get_encrypt_flags(u_int32_t *);
-	virtual int set_encrypt(const char *, u_int32_t);
-	virtual void set_errcall(void (*)(const char *, char *));
-	virtual void get_errfile(FILE **);
-	virtual void set_errfile(FILE *);
-	virtual void get_errpfx(const char **);
-	virtual void set_errpfx(const char *);
-	virtual int set_feedback(void (*)(Db *, int, int));
-	virtual int get_flags(u_int32_t *);
-	virtual int set_flags(u_int32_t);
-	virtual int get_h_ffactor(u_int32_t *);
-	virtual int set_h_ffactor(u_int32_t);
-	virtual int set_h_hash(h_hash_fcn_type); /*deprecated*/
-	virtual int set_h_hash(u_int32_t (*)(Db *, const void *, u_int32_t));
-	virtual int get_h_nelem(u_int32_t *);
-	virtual int set_h_nelem(u_int32_t);
-	virtual int get_lorder(int *);
-	virtual int set_lorder(int);
-	virtual int get_pagesize(u_int32_t *);
-	virtual int set_pagesize(u_int32_t);
-	virtual int set_paniccall(void (*)(DbEnv *, int));
-	virtual int get_re_delim(int *);
-	virtual int set_re_delim(int);
-	virtual int get_re_len(u_int32_t *);
-	virtual int set_re_len(u_int32_t);
-	virtual int get_re_pad(int *);
-	virtual int set_re_pad(int);
-	virtual int get_re_source(const char **);
-	virtual int set_re_source(const char *);
-	virtual int get_q_extentsize(u_int32_t *);
-	virtual int set_q_extentsize(u_int32_t);
-	virtual int stat(void *sp, u_int32_t flags);
-	virtual int sync(u_int32_t flags);
-	virtual int truncate(DbTxn *, u_int32_t *, u_int32_t);
-	virtual int upgrade(const char *name, u_int32_t flags);
-	virtual int verify(const char *, const char *, __DB_OSTREAMCLASS *,
-		      u_int32_t);
+	void set_app_private(void *);
+	int set_append_recno(int (*)(Db *, Dbt *, db_recno_t));
+	int set_bt_compare(bt_compare_fcn_type); /*deprecated*/
+	int set_bt_compare(int (*)(Db *, const Dbt *, const Dbt *));
+	int set_bt_maxkey(u_int32_t);
+	int set_bt_minkey(u_int32_t);
+	int set_bt_prefix(bt_prefix_fcn_type); /*deprecated*/
+	int set_bt_prefix(size_t (*)(Db *, const Dbt *, const Dbt *));
+	int set_cachesize(u_int32_t, u_int32_t, int);
+	int set_dup_compare(dup_compare_fcn_type); /*deprecated*/
+	int set_dup_compare(int (*)(Db *, const Dbt *, const Dbt *));
+	void set_errcall(void (*)(const char *, char *));
+	void set_errfile(FILE *);
+	void set_errpfx(const char *);
+	int set_feedback(void (*)(Db *, int, int));
+	int set_flags(u_int32_t);
+	int set_h_ffactor(u_int32_t);
+	int set_h_hash(h_hash_fcn_type); /*deprecated*/
+	int set_h_hash(u_int32_t (*)(Db *, const void *, u_int32_t));
+	int set_h_nelem(u_int32_t);
+	int set_lorder(int);
+	int set_pagesize(u_int32_t);
+	int set_paniccall(void (*)(DbEnv *, int));
+	int set_re_delim(int);
+	int set_re_len(u_int32_t);
+	int set_re_pad(int);
+	int set_re_source(char *);
+	int set_q_extentsize(u_int32_t);
+	int stat(void *sp, u_int32_t flags);
+	int sync(u_int32_t flags);
+	int truncate(DbTxn *, u_int32_t *, u_int32_t);
+	int upgrade(const char *name, u_int32_t flags);
+	int verify(const char *, const char *, OSTREAMCLASS *, u_int32_t);
 
 	// These additional methods are not in the C interface, and
 	// are only available for C++.
 	//
-	virtual void set_error_stream(__DB_OSTREAMCLASS *);
+	void set_error_stream(OSTREAMCLASS *);
 
-	virtual DbEnv *get_env();
-	virtual DbMpoolFile *get_mpf();
-
-	virtual DB *get_DB()
+	DB *get_DB()
 	{
 		return (DB *)imp();
 	}
 
-	virtual const DB *get_const_DB() const
+	const DB *get_const_DB() const
 	{
 		return (const DB *)constimp();
 	}
 
 	static Db* get_Db(DB *db)
 	{
-		return (Db *)db->api_internal;
+		return (Db *)db->cj_internal;
 	}
 
 	static const Db* get_const_Db(const DB *db)
 	{
-		return (const Db *)db->api_internal;
+		return (const Db *)db->cj_internal;
 	}
 
+	// These are public only because they need to be called
+	// via C callback functions.  They should never be used by
+	// external users of this class.
+	//
+	void (*feedback_callback_)(Db *, int, int);
+	int (*append_recno_callback_)(Db *, Dbt *, db_recno_t);
+	int (*bt_compare_callback_)(Db *, const Dbt *, const Dbt *);
+	size_t (*bt_prefix_callback_)(Db *, const Dbt *, const Dbt *);
+	int (*dup_compare_callback_)(Db *, const Dbt *, const Dbt *);
+	u_int32_t (*h_hash_callback_)(Db *, const void *, u_int32_t);
+	int (*associate_callback_)(Db *, const Dbt *, const Dbt *, Dbt *);
 private:
+
 	// no copying
 	Db(const Db &);
 	Db &operator = (const Db &);
 
+	DEFINE_DB_CLASS(Db);
+
 	void cleanup();
 	int initialize();
 	int error_policy();
 
 	// instance data
 	DbEnv *env_;
-	DbMpoolFile *mpf_;
 	int construct_error_;
 	u_int32_t flags_;
 	u_int32_t construct_flags_;
-
-public:
-	// These are public only because they need to be called
-	// via C callback functions.  They should never be used by
-	// external users of this class.
-	//
-	int (*append_recno_callback_)(Db *, Dbt *, db_recno_t);
-	int (*associate_callback_)(Db *, const Dbt *, const Dbt *, Dbt *);
-	int (*bt_compare_callback_)(Db *, const Dbt *, const Dbt *);
-	size_t (*bt_prefix_callback_)(Db *, const Dbt *, const Dbt *);
-	int (*dup_compare_callback_)(Db *, const Dbt *, const Dbt *);
-	void (*feedback_callback_)(Db *, int, int);
-	u_int32_t (*h_hash_callback_)(Db *, const void *, u_int32_t);
 };
 
 //
@@ -938,7 +657,7 @@ public:
 	static const Dbt* get_const_Dbt(const DBT *dbt)
 					       { return (const Dbt *)dbt; }
 
-	Dbt(void *data, u_int32_t size);
+	Dbt(void *data, size_t size);
 	Dbt();
 	~Dbt();
 	Dbt(const Dbt &);
@@ -1004,44 +723,4 @@ private:
 	DbLogc(const Dbc &);
 	DbLogc &operator = (const Dbc &);
 };
-
-////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////
-//
-// multiple key/data/reco iterator classes
-//
-
-// DbMultipleIterator is a shared private base class for the three types
-// of bulk-return Iterator;  it should never be instantiated directly,
-// but it handles the functionality shared by its subclasses.
-class _exported DbMultipleIterator
-{
-public:
-	DbMultipleIterator(const Dbt &dbt);
-protected:
-	u_int8_t *data_;
-	u_int32_t *p_;
-};
-
-class _exported DbMultipleKeyDataIterator : private DbMultipleIterator
-{
-public:
-	DbMultipleKeyDataIterator(const Dbt &dbt) : DbMultipleIterator(dbt) {}
-	bool next(Dbt &key, Dbt &data);
-};
-
-class _exported DbMultipleRecnoDataIterator : private DbMultipleIterator
-{
-public:
-	DbMultipleRecnoDataIterator(const Dbt &dbt) : DbMultipleIterator(dbt) {}
-	bool next(db_recno_t &recno, Dbt &data);
-};
-
-class _exported DbMultipleDataIterator : private DbMultipleIterator
-{
-public:
-	DbMultipleDataIterator(const Dbt &dbt) : DbMultipleIterator(dbt) {}
-	bool next(Dbt &data);
-};
-
 #endif /* !_DB_CXX_H_ */
